using System;
using System.Collections.Generic;
using System.Data;
using System.Data.OleDb;
using Isep.Util.Data;

namespace EntityManagement.ActiveRecordFramework
{
	public abstract class ActiveRecord
	{
		public ActiveRecord()
		{
            SetDirty();
		}

        internal ActiveRecord(DataRow r)
        {
            myID = (int)r["id"];
        }

		protected int? myID;

		public int ID
		{
			get
			{
                if (!myID.HasValue)
                    throw new InvalidOperationException();

				return myID.Value;
			}
		}

        //private bool isNew;
        public bool IsNew
        {
            get { return myID.HasValue == false; }
        }

        private bool isDirty;
        public bool IsDirty
        {
            get { return isDirty;  }
        }

        protected void SetDirty()
        {
            isDirty = true;
        }

        private bool isDeletedOrInvalid;

        internal void Invalidate()
        {
            isDeletedOrInvalid = true;
        }

        protected virtual void OnInserted() { }
        protected virtual void OnUpdated() { }
        protected virtual void OnBeforeSave() { }
        protected virtual void OnSaved() { }

        public virtual void Save()
        {
            if (isDeletedOrInvalid)
                throw new InvalidOperationException("Deleted or discarted object cannot perform this operation");

            if (!isDirty)
                return;

            OnBeforeSave();

            IDictionary<string, object> parms = GetParameterValues();

            OleDbCommand cmd = new OleDbCommand();
            cmd.Connection = OleDbHelper.CreateOpenConnection(Properties.Settings.Default.ConnStr);

            foreach (string fieldname in parms.Keys)
            {
                if (parms[fieldname] == null)
                    cmd.Parameters.AddWithValue("@" + fieldname, DBNull.Value);
                else
                    cmd.Parameters.AddWithValue("@" + fieldname, parms[fieldname]);
            }

            System.Text.StringBuilder sb = new System.Text.StringBuilder();
            if (IsNew)
            {
                sb.Append("INSERT INTO ");
                sb.Append(TableName);
                sb.Append(" (");
                foreach (string fieldname in parms.Keys)
                {
                    sb.Append(fieldname);
                    sb.Append(", ");
                }
                sb.Length = sb.Length - 2;
                //designacao, virtual
                sb.Append(") VALUES(");
                foreach (string fieldname in parms.Keys)
                    sb.Append("?, ");
                sb.Length = sb.Length - 2;
                sb.Append(")");
                
                cmd.CommandText = sb.ToString();
                cmd.ExecuteNonQuery();

                cmd.CommandText = "SELECT @@IDENTITY";
                cmd.Parameters.Clear();
                myID = (int)cmd.ExecuteScalar();

                Finder.RegisterInserted(this);

                OnInserted();
            }
            else
            {
                sb.Append("UPDATE ");
                sb.Append(TableName);
                sb.Append(" SET ");
                foreach (string fieldname in parms.Keys)
                {
                    sb.Append(fieldname);
                    sb.Append("=?, ");
                }
                sb.Length = sb.Length - 2;
                sb.Append(" WHERE id=?");

                cmd.CommandText = sb.ToString();

                cmd.Parameters.AddWithValue("@id", ID);
                cmd.ExecuteNonQuery();

                OnUpdated();
            }
            cmd.Connection.Close();

            OnSaved();
        }

        protected abstract IDictionary<string, object> GetParameterValues();
        internal abstract string TableName { get; }
        //protected virtual string AdditionalFilter { get { return "1=1"; } }

        public virtual void Delete()
        {
            if (isDeletedOrInvalid)
                throw new InvalidOperationException("Deleted or discarted object cannot perform this operation.");

            if (IsNew)
                throw new InvalidOperationException("Cannot delete an object that has not been saved.");

            OleDbHelper.ExecuteUpdate(Properties.Settings.Default.ConnStr, string.Format("DELETE FROM {0} WHERE Id={1}", TableName, ID));

            Finder.RegisterDeleted(this);
            myID = null;

            Invalidate();
        }


        internal virtual void Refresh(DataRow r)
        {
            if (IsNew)
                throw new InvalidOperationException();

            if ((int)r["ID"] != myID)
                throw new InvalidOperationException("Cannot refresh from a diferent row.");
        }

#region DAL
        //protected static string CONNSTR
        //{
        //    get 
        //    {
        //        return System.Configuration.ConfigurationSettings.AppSettings["ConnStr"];
        //    }
        //}

        //private static OleDbTransaction myTx;
        //private static OleDbConnection myCnx;

        //protected static OleDbTransaction CurrentTransaction
        //{
        //    get { return myTx; }
        //}

        //protected static OleDbConnection GetConnection(bool open)
        //{
        //    OleDbConnection cnx = new OleDbConnection(CONNSTR);
        //    if (open)
        //        cnx.Open();
        //    return cnx;
        //}

        //protected static DataSet ExecuteQuery(OleDbConnection cnx, string sql)
        //{
        //    try
        //    {
        //        OleDbDataAdapter da = new OleDbDataAdapter(sql, cnx);
        //        DataSet ds = new DataSet();
        //        da.Fill(ds);
        //        return ds;
        //    }
        //    catch (OleDbException ex)
        //    {
        //        throw new ApplicationException("Erro BD", ex);
        //    }
        //}

        //protected static int ExecuteNonQuery(OleDbConnection cnx, string sql)
        //{
        //    OleDbCommand cmd = new OleDbCommand(sql, cnx);
        //    return cmd.ExecuteNonQuery();
        //}

        //protected static int ExecuteNonQuery(OleDbTransaction tx, string sql)
        //{
        //    OleDbCommand cmd = new OleDbCommand(sql, tx.Connection, tx);
        //    return cmd.ExecuteNonQuery();
        //}

        //protected static int ExecuteNonQuery(OleDbTransaction tx, OleDbCommand cmd)
        //{
        //    cmd.Transaction = tx;
        //    return cmd.ExecuteNonQuery();
        //}

        //protected static DataSet ExecuteQuery(string sql)
        //{
        //    try
        //    {
        //        if (myCnx == null)
        //            myCnx = GetConnection(false);

        //        OleDbDataAdapter da = new OleDbDataAdapter(sql, myCnx);
        //        DataSet ds = new DataSet();
        //        da.Fill(ds);
        //        return ds;
        //    }
        //    catch (OleDbException ex)
        //    {
        //        throw new ApplicationException("Erro BD", ex);
        //    }
        //}

        //protected static DataSet ExecuteTransactedQuery(string sql)
        //{
        //    try
        //    {
        //        OleDbDataAdapter da = new OleDbDataAdapter(sql, myCnx);
        //        DataSet ds = new DataSet();
        //        da.Fill(ds);
        //        return ds;
        //    }
        //    catch (OleDbException ex)
        //    {
        //        throw new ApplicationException("Erro BD", ex);
        //    }
        //}

        //protected static int ExecuteNonQuery(string sql)
        //{
        //    OleDbConnection cnx = GetConnection(true);
        //    int r = ExecuteNonQuery(cnx, sql);
        //    cnx.Close();
        //    return r;
        //}

        //protected static int ExecuteTransactedNonQuery(string sql)
        //{
        //    OleDbCommand cmd = new OleDbCommand(sql);
        //    return ExecuteTransactedNonQuery(cmd);
        //}

        //protected static int ExecuteTransactedNonQuery(OleDbCommand cmd)
        //{
        //    try
        //    {
        //        cmd.Transaction = CurrentTransaction;
        //        cmd.Connection = CurrentTransaction.Connection;
        //        return cmd.ExecuteNonQuery();
        //    }
        //    catch (OleDbException ex)
        //    {
        //        // debug purposes only!!!
        //        throw ex;	
        //    }
        //}

        //protected static void BeginTransaction()
        //{
        //    try
        //    {
        //        if (myTx == null)
        //            myTx = GetConnection(true).BeginTransaction();
        //    }
        //    catch (OleDbException ex)
        //    {
        //        throw new ApplicationException("Erro BD", ex);
        //    }
        //}

        //protected static void CommitTransaction()
        //{
        //    if (myTx != null)
        //    {
        //        OleDbConnection cnx = myTx.Connection;
        //        myTx.Commit();
        //        cnx.Close();
        //    }
        //}

        //protected static void RoolbackTransaction()
        //{
        //    if (myTx != null)
        //    {
        //        OleDbConnection cnx = myTx.Connection;
        //        myTx.Rollback();
        //        cnx.Close();
        //    }
        //}
#endregion
    }
}
