using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Data.OleDb;
using Isep.Util.Data;

namespace EntityManagement.ActiveRecordFramework
{
    public abstract class Finder
    {
        // implementa Identity Map
        /*
         * de notar que esta implementao s  vlida para ambientes tipo web/servidor de aplicaes
         * em que cada invocao do programa cliente provoca a instanciao do componente
         * em ambientes stand-alone, como o identity map  static ao fim de algum tempo toda
         * a base de dados vai estar em memria
         * poderia ser resolvido usando um par de mtodos para indicar o inicio e fim de "transaco de negcio"
         * para limpar o identity map quando no fosse necessrio (ver tb. padro Unit Of Work)
         */
        private static IDictionary<string, IDictionary<int, ActiveRecord>> _loaded = new Dictionary<string, IDictionary<int, ActiveRecord>>();
        //private static IDictionary<int, ActiveRecord> _loaded = new Dictionary<int, ActiveRecord>();

        //public static void BeginBusinessTransaction()
        //{
        //    // assure that any reference to these objects will result in error
        //    foreach (IDictionary<int, ActiveRecord> k in _loaded.Values)
        //        foreach (ActiveRecord o in k.Values)
        //            o.Invalidate();

        //    _loaded.Clear();
        //}

        //public static void EndBusinessTransaction()
        //{
        //    // assure that any reference to these objects will result in error
        //    foreach (IDictionary<int, ActiveRecord> k in _loaded.Values)
        //        foreach (ActiveRecord o in k.Values)
        //        {
        //            //o.Commit();
        //            o.Invalidate();
        //        }

        //    _loaded.Clear();
        //}

        // para os active records registarem os objectos inseridos (ver Save())
        internal static void RegisterInserted(ActiveRecord obj)
        {
            RegisterLoaded(obj.TableName, obj.ID, obj);
        }

        protected virtual ActiveRecord LoadById(int id)
        {
            if (PreviouslyLoaded(id))
                return IsLoaded(id);

            ActiveRecord obj = CreateFromData(LoadDataRowById(id));

            RegisterLoaded(id, obj);

            return obj;
        }

        private IList<ActiveRecord> CreateFromDataTable(DataTable dt)
        {
            IList<ActiveRecord> ret = new List<ActiveRecord>();
            ActiveRecord obj;

            foreach (DataRow r in dt.Rows)
            {
                if (PreviouslyLoaded((int)r["id"]))
                {
                    obj = IsLoaded((int)r["id"]);
                    obj.Refresh(r);
                }
                else
                {
                    obj = CreateFromData(r);
                    RegisterLoaded((int)r["id"], obj);
                }
                ret.Add(obj);
            }

            return ret;
        }

        protected virtual IList<ActiveRecord> LoadByStringField(string fieldName, string fieldValue, bool exact)
        {
            return CreateFromDataTable(LoadDataTable(fieldName, fieldValue, exact));
        }

        protected virtual IList<ActiveRecord> LoadByForeignKey(string fieldName, int fieldValue)
        {
            return CreateFromDataTable(LoadDataTable(fieldName, fieldValue));
        }

        protected virtual IList<ActiveRecord> LoadAll()
        {
            return CreateFromDataTable(LoadDataTable());
        }

        private void RegisterLoaded(int id, ActiveRecord obj)
        {
            RegisterLoaded(TableName, id, obj);
        }

        private static void RegisterLoaded(string tableName, int id, ActiveRecord obj)
        {
            if (_loaded.ContainsKey(tableName))
            {
                IDictionary<int, ActiveRecord> k = _loaded[tableName];
                k.Add(id, obj);
            }
            else
            {
                IDictionary<int, ActiveRecord> k = new Dictionary<int, ActiveRecord>();
                k.Add(id, obj);
                _loaded.Add(tableName, k);
            }
        }

        private bool PreviouslyLoaded(int id)
        {
            if (_loaded.ContainsKey(TableName))
            {
                IDictionary<int, ActiveRecord> k = _loaded[TableName];
                return k.ContainsKey(id);
            }
            else
                return false;
        }

        private ActiveRecord IsLoaded(int id)
        {
            IDictionary<int, ActiveRecord> k = _loaded[TableName];
            return k[id];
        }

        protected abstract ActiveRecord CreateFromData(DataRow dataRow);

        protected virtual DataRow LoadDataRowById(int id)
        {
            string sql = string.Format("SELECT * FROM {0} WHERE id={1} AND {2}", TableName, id, AdditionalFilter);
            DataSet ds = OleDbHelper.ExecuteDataSet(Properties.Settings.Default.ConnStr, sql);
            if (ds.Tables[0].Rows.Count == 0)
                return null;

            return ds.Tables[0].Rows[0];
        }

        protected virtual DataTable LoadDataTable()
        {
            string sql = string.Format("SELECT * FROM {0} WHERE {1}", TableName, AdditionalFilter);
            DataSet ds = OleDbHelper.ExecuteDataSet(Properties.Settings.Default.ConnStr, sql);

            return ds.Tables[0];
        }

        protected virtual DataTable LoadDataTable(string fieldName, string fieldValue, bool exact)
        {
            string sql;
            if (exact)
                sql = string.Format("SELECT * FROM {0} WHERE {1} = '{2}' AND {3}", TableName, fieldName, fieldValue, AdditionalFilter);
            else
                sql = string.Format("SELECT * FROM {0} WHERE {1} LIKE '{2}%' AND {3}", TableName, fieldName, fieldValue, AdditionalFilter);
            DataSet ds = OleDbHelper.ExecuteDataSet(Properties.Settings.Default.ConnStr, sql);

            return ds.Tables[0];
        }

        protected virtual DataTable LoadDataTable(string fieldName, int fieldValue)
        {
            string sql = string.Format("SELECT * FROM {0} WHERE {1} = {2} AND {3}", TableName, fieldName, fieldValue, AdditionalFilter);
            DataSet ds = OleDbHelper.ExecuteDataSet(Properties.Settings.Default.ConnStr, sql);

            return ds.Tables[0];
        }

        protected abstract string TableName { get; }
        protected virtual string AdditionalFilter { get { return "1=1"; } }

        internal static void RegisterDeleted(ActiveRecord obj)
        {
            IDictionary<int, ActiveRecord> k = _loaded[obj.TableName];
            k.Remove(obj.ID);
        }
    }
}
