using System;
using System.Collections.Generic;
using System.Text;

using EntityManagement.ActiveRecordFramework;

namespace EntityManagement
{
    public class Address : ActiveRecord
    {
        string street;

        public string Street
        {
            get { return street; }
            set
            {
                if (value == null)
                    throw new ArgumentNullException();
                street = value;

                SetDirty();
            }
        }

        string street2;

        public string Street2
        {
            get { return street2; }
            set
            {
                street2 = value;
                SetDirty();
            }
        }

        string local;

        public string Local
        {
            get { return local; }
            set
            {
                local = value;
                SetDirty();
            }
        }

        Country country;

        public Country Country
        {
            get { return country; }
            set 
            {
                if (value == null)
                    throw new ArgumentNullException();
                country = value;

                SetDirty();
            }
        }

        IList<AddressOfEntity> addressees; // moradores

        public IList<AddressOfEntity> Addressees
        {
            get 
            {
                EnsureLoadedAddressees();

                return new System.Collections.ObjectModel.ReadOnlyCollection<AddressOfEntity>(addressees);
            }
            //set { addressees = value; }
        }

        private void EnsureLoadedAddressees()
        {
            if (addressees == null)
            {
                if (IsNew)
                    addressees = new List<AddressOfEntity>();
                else
                    addressees = new AddressOfEntityFinder().GetByAddress(this /*ID*/);
            }
        }

        public void AddAddressee(Entity e)
        {
            EnsureLoadedAddressees();

            AddressOfEntity aoe = new AddressOfEntity(this, e);
            addressees.Add(aoe);

            e.AddAddress(aoe);

            SetDirty();
        }

        public Address()
            : base()
        {
        }

        public Address(string s, string s2, string l, Country c)
            : base()
        {
            this.Street = s;
            this.Street2 = s2;
            this.Local = l;
            this.Country = c;
        }

        #region internal methods

        internal void AddAddressee(AddressOfEntity aoe)
        {
            EnsureLoadedAddressees();
            addressees.Add(aoe);
        }
        
#endregion

        #region DAL

        protected override IDictionary<string, object> GetParameterValues()
        {
            IDictionary<string, object> parms = new Dictionary<string, object>();
            parms.Add("street", street);
            parms.Add("street2", street2);
            parms.Add("localArea", local);
            parms.Add("idcountry", country.ID);
            return parms;
        }

        internal override string TableName
        {
            get { return TableNames.Addresses; }
        }

        internal Address(System.Data.DataRow r)
            : base(r)
        {
            Street = (string)r["street"];
            Street2 = (string)r["street2"];
            Local = (string)r["localArea"];
            Country = new CountryFinder().GetById( (int)r["idcountry"] );
        }

        #endregion
    }
}
