using System;
using System.Collections.Generic;
using System.Text;

using EntityManagement.ActiveRecordFramework;

namespace EntityManagement
{
    public abstract class Entity : ActiveRecord
    {
        string name;

        public string Name
        {
            get { return name; }
            set
            {
                if (value == null)
                    throw new ArgumentNullException();
                name = value;

                SetDirty();
            }
        }

        string nif;

        public string Nif
        {
            get { return nif; }
            set
            {
                nif = value;
                SetDirty();
            }
        }

        IList<AddressOfEntity> addresses; // moradas

        public IList<AddressOfEntity> Addresses
        {
            get 
            {
                EnsureLoadedAddresses();

                return new System.Collections.ObjectModel.ReadOnlyCollection<AddressOfEntity>(addresses); 
            }

            // no SET for this property
            //set { addresses = value; }
        }

        private void EnsureLoadedAddresses()
        {
            if (addresses == null)
            {
                if (IsNew)
                    addresses = new List<AddressOfEntity>(); 
                else
                    addresses = new AddressOfEntityFinder().GetByEntity(this /*ID*/);
            }
        }

        public void AddAddress(Address addr)
        {
            EnsureLoadedAddresses();

            AddressOfEntity aoe = new AddressOfEntity(addr, this);
            addresses.Add(aoe);

            addr.AddAddressee(aoe);

            SetDirty();
        }

        public Entity()
            : base()
        {
        }

        #region internal methods

        internal void AddAddress(AddressOfEntity aoe)
        {
            EnsureLoadedAddresses();
            addresses.Add(aoe);

            SetDirty();
        }

        #endregion

        #region DAL

        protected override void OnSaved()
        {
            EnsureLoadedAddresses();
            foreach (AddressOfEntity aoe in addresses)
                aoe.Save();
        }

        protected override IDictionary<string, object> GetParameterValues()
        {
            IDictionary<string, object> parms = new Dictionary<string, object>();
            parms.Add("name", name);
            parms.Add("nif", nif);
            if (this is Person)
                parms.Add("type", 0);
            else
                parms.Add("type", 1);
            return parms;
        }

        internal Entity(System.Data.DataRow r) : base(r)
        {
            name = (string)r["name"];
            nif = (string)r["nif"];
        }
        #endregion
    }
}
